<?php

/**
 * @file
 * Contains batch functions for importing nodes.
 */

/**
 * Batch operation callback for processing nodes.
 *
 * Uses Transport module (and thus Clients too) to retrieve remote nodes and their dependencies.
 */
function _uprpc_batch_nodes(&$context) {
  $limit = 5;

  module_load_include("inc", "uprpc");

  // Get the content types as saved on the admin settings form.
  // This is an array of the form $type_remote => $type_local.
  $content_types = variable_get('uprpc_node_types', array());

  // Node import works a little differently from users and terms.
  // Some content types should not ALL be migrated.
  // We invoke hook_nodeimport_query() (see that for note about reworking needed!)
  // to give us extra JOIN and WHERE clauses for the queries, and thus
  // we alter the count and list queries to send to the remote site for each type.
  // However, this means we get the WHOLE list of nids at the start of the batch
  // and store it as a big fat array of nids in the batch sandbox.
  // I am reliable told on IRC that an array of some 8000 nodes ids should
  // be manageable.
  // The altermative would be to muck around either creating one batch operation
  // for each node type, or being quite creative in getting the next
  // set of 10 nids to work on, and probably storing an array of types,
  // working on nids from the first one, popping it off the array once we 
  // have no more, etc etc.

  // Invoke hook_nodeimport_query() to see if there are any changes to be made to the queries for these types.
  // @see the query built in uprpc_services_get_import_list().
  $query_data = module_invoke_all('nodeimport_query');
  foreach ($content_types as $type_remote => $type_local) {
    $joins[$type_remote] = ' '; // always have to send at least a space; Services chokes on empty string.
    $where[$type_remote] = " b.type = '$type_remote' ";
    if (isset($query_data[$type_remote])) {
      $joins[$type_remote] .= $query_data[$type_remote]['joins'];
      $where[$type_remote] .= ' AND ' . $query_data[$type_remote]['where'];
    }
  }

  if (!$context['sandbox']) {
    // First time here: set up.

    // Set up our connection.
    $connection_id = variable_get('uprpc_client_connection', 0);
    $drupal_client = clients_get_connection($connection_id);
    dsm($drupal_client);
    
    // Get the setting for what to retrieve: everything, only new nodes, or new and updated nodes.
    $node_retrieve_filter = variable_get('uprpc_remote_filter', 'new');    
    
    // Get the total number of nodes to import.
    // TODO: is this actually needed at all???
    foreach ($content_types as $type_remote => $type_local) {
      //$max_type = _uprpc_xmlrpc('query.count_unimported', 'node', 'nid', $joins[$type], $where[$type]);
      $max_type = $drupal_client->callMethod('query.count_unimported', 'node', 'nid', $joins[$type_remote], $where[$type_remote], $node_retrieve_filter);
      $max += $max_type;
    }
    
    if (!$max) {
      drupal_set_message("No nodes to process: have we already run the import?");
      $context['finished'] = 1;
      return;
    }

    // Build a list of all the node ids we need to process.
    $list = array();
    foreach ($content_types as $type_remote => $type_local) {
      //$list_type = _uprpc_xmlrpc('query.get_import_list', 'node', 'nid', 0, $joins[$type], $where[$type]);
      $list_type = $drupal_client->callMethod('query.get_import_list', 'node', 'nid', 0, $joins[$type_remote], $where[$type_remote], $node_retrieve_filter);
      // The remote method SHOULD be always returning at least an empty array,
      // but due to WTF reasons is not :/
      if (is_array($list_type)) {
        $list = array_merge($list, $list_type);
      }
    }
    
    //dsm($max_type, 'max type');
    //dsm($list, 'list');
    
    // Abort the batch if there is nothing to retrieve.
    if (!count($list)) {
      drupal_set_message("No nodes to process: have we already run the import?");
      $context['finished'] = 1;
      return;
    }
    
    // Create our transport controller, passing our (arbitrary) name to identify the source.
    $options = array(
      'operation_limit' => $limit,
    );
    $transport_controller = new TransportControllerRetriever(UPRPC_TRANSPORT_SOURCE_NAME, $drupal_client, $options);
    // Make up the list to pass to the controller.
    $entities_list = array(
      'node' => $list,
    );
    // Add our list to the controller.
    $transport_controller->addEntityList($entities_list);
    dsm($transport_controller, 'tc');

    $context['sandbox']['source'] = $drupal_client;
    $context['sandbox']['controller'] = $transport_controller;
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['max'] = $max;
  }
  else {
    // Been here before: retrieve things from storage.
    $transport_controller = $context['sandbox']['controller'];
    $drupal_client        = $context['sandbox']['source'];
  }

  // Start a retrieval. The process limit internal to the controller will
  // only process as many as we set in the options array.
  $processed_list = $transport_controller->beginRetrieval();
  
  // Mark imported nodes on the remote site.
  if (variable_get('uprpc_remote_mark', TRUE)) {
    foreach ($processed_list as $processed_item) {
      $drupal_client->callMethod('query.set_imported', $processed_item['type'], (int) $processed_item['id']);
    }
  }
    
  $processed = $transport_controller->count_entities_processed();
  $remaining = $transport_controller->count_entities_remaining();
  $total     = $processed + $remaining;
  
  $context['sandbox']['progress'] = $processed;
  $context['sandbox']['max']      = $total;
  
  // Batch API progress
  $context['message'] = t("@count nodes processed out of @total", array(
    '@count' => $processed,
    '@total' => $total,
  ));

  $context['finished'] = $processed / $total;
  
  // TODO: if context finished, get counts and stuff from the controller and produce messages.
}
